(c) C. Alex McLean 2007
Released under GPL version 2.  See the COPYING file or gnu.org for details.

> module KarplusStrong where

> import Sound.OpenSoundControl
> import Sound.SC3
> import Data.List
> import Control.Monad
> import Control.Concurrent
> import System.Directory
> import System.Random
> import Text.Printf
> import Char


> initSynth :: Int -> IO ()
> initSynth 0 = return ()
> initSynth voices = do withSC3 reset
>                       -- fix from rohan, make sure control buses are
>                       -- initialised.  i'll do this properly one day...
>                       mapM ((flip playWord) "test") [0..voices-1]
>                       initVoices voices

> initVoices :: Int -> IO ()
> initVoices 0 = return ()
> initVoices voices = do synth <- ks $ voices - 1
>                        audition $ synth
>                        putStrLn $ "initialised " ++ (show (voices - 1))
>                        initVoices $ voices - 1

> putnam = [ -- soprano
>            ( "sA" 
>            , [800, 1150, 2900, 3900, 4950]
>            , [0, -6,  -32, -20, -50]
>            , [80, 90,  120, 130, 140] )
>          , ( "sE" 
>            , [350, 2000, 2800, 3600, 4950]
>            , [0, -20, -15, -40, -56]
>            , [60, 100, 120, 150, 200] )
>          , ( "sI" 
>            , [270, 2140, 2950, 3900, 4950]
>            , [0, -12 , -26 , -26 , -44]
>            , [60, 90,  100, 120, 120] )
>          , ( "sO" 
>            , [450, 800,  2830, 3800, 4950]
>            , [0, -11 , -22 , -22 , -50]
>            , [70, 80,  100, 130, 135] )
>          , ( "sU" 
>            , [325, 700,  2700, 3800, 4950]
>            , [0, -16, -35, -40, -60]
>            , [50, 60,  170, 180, 200] )
>            -- alto 
>          , ( "aA" 
>            , [800, 1150, 2800, 3500, 4950]
>            , [0, -4  , -20, -36 , -60]
>            , [80, 90,  120, 130, 140] )
>          , ( "aE" 
>            , [400, 1600, 2700, 3300, 4950]
>            , [0, -24 , -30, -35, -60]
>            , [60, 80,  120, 150, 200] )
>          , ( "aI" 
>            , [350, 1700, 2700, 3700, 4950]
>            , [0, -20, -30, -36 , -60]
>            , [50, 100, 120, 150, 200] )
>          , ( "aO" 
>            , [450, 800,  2830, 3500, 4950]
>            , [0, -9  , -16 , -28 , -55]
>            , [70, 80,  100, 130, 135] )
>          , ( "aU" 
>            , [325, 700, 2530, 3500, 4950]
>            , [0, -12 , -30, -40, -64]
>            , [50, 60,  170, 180, 200] )
>            -- counter tenor
>          , ( "ctA"
>            , [660, 1120, 2750, 3000, 3350]
>            , [0, -6  , -23 , -24 , -38]
>            , [80, 90,  120, 130, 140] )
>          , ( "ctE"
>            , [440, 1800, 2700, 3000, 3300]
>            , [0, -14 , -18 , -20, -20]
>            , [70, 80,  100, 120, 120] )
>          , ( "ctI"
>            , [270, 1850, 2900, 3350, 3590]
>            , [0, -24 , -24 , -36 , -36]
>            , [40, 90,  100, 120, 120] )
>          , ( "ctO"
>            , [430, 820,  2700, 3000, 3300]
>            , [0, -10, -26 , -22 , -34]
>            , [40, 80,  100, 120, 120] )
>          , ( "ctU"
>            , [370, 630,  2750, 3000, 3400]
>            , [0, -20, -23 , -30, -34]
>            , [40, 60,  100, 120, 120] )
>            -- tenor
>          , ( "tA" 
>            , [650, 1080, 2650, 2900, 3250]
>            , [0, -6   , -7  , -8 , -22]
>            , [80, 90,  120, 130, 140] )
>          , ( "tE" 
>            , [400, 1700, 2600, 3200, 3580]
>            , [0, -14 , -12 , -14 , -20]
>            , [70, 80,  100, 120, 120] )
>          , ( "tI" 
>            , [290, 1870, 2800, 3250, 3540]
>            , [0, -15, -18 , -20, -30]
>            , [40, 90,  100, 120, 120] )
>          , ( "tO" 
>            , [400, 800,  2600, 2800, 3000]
>            , [0, -10, -12 , -12 , -26]
>            , [40, 80,  100, 120, 120] )
>          , ( "tU" 
>            , [350, 600,  2700, 2900, 3300]
>            , [0, -20, -17 , -14 , -26]
>            , [40, 60,  100, 120, 120] )
>            -- bass
>          , ( "bA" 
>            , [600, 1040, 2250, 2450, 2750]
>            , [0, -7   , -9  , -9 , -20]
>            , [60, 70,  110, 120, 130] )
>          , ( "bE" 
>            , [400, 1620, 2400, 2800, 3100]
>            , [0, -12  , -9 , -12 , -18]
>            , [40, 80,  100, 120, 120] )
>          , ( "bI" 
>            , [250, 1750, 2600, 3050, 3340]
>            , [0, -30, -16 , -22 , -28]
>            , [60, 90,  100, 120, 120] )
>          , ( "bO" 
>            , [400, 750,  2400, 2600, 2900]
>            , [0, -11 , -21 , -20, -40]
>            , [40, 80,  100, 120, 120] )
>          , ( "bU" 
>            , [350, 600,  2400, 2675, 2950]
>            , [0, -20, -32 , -28 , -36]
>            , [40, 80,  100, 120, 120] ) ]

> sendVowel i str 
>               = withSC3 $ \fd -> (send fd $ c_setn [(offset i 0, f), 
>                                                     (offset i 5, a),
>                                                     (offset i 10, b)
>                                                    ]
>                                  )
>     where (f, a, b) = findStr putnam
>           findStr ((str', f, a, b):xs) | str == str' = (f, a, b)
>                                        | otherwise = findStr xs

> consonants = [('b', 0.00223, 1.00000),
>               ('c', 0.00317, 0.86667),
>               ('d', 0.00596, 0.00571),
>               ('f', 0.00105, 0.89714),
>               ('g', 0.00340, 0.95330),
>               ('h', 0.00474, 0.99619),
>               ('j', 0.00187, 0.90000),
>               ('k', 0.00310, 0.47238),
>               ('l', 0.00315, 0.95333),
>               ('m', 0.00389, 1.00000),
>               ('n', 0.00384, 0.99905),
>               ('p', 0.00572, 0.99810),
>               ('q', 0.00983, 0.08762),
>               ('r', 0.00106, 0.93619),
>               ('s', 0.00103, 0.88286),
>               ('t', 0.00125, 0.12762),
>               ('v', 0.00359, 0.00095),
>               ('w', 0.00940, 1.00000),
>               ('x', 0.00550, 0.51905),
>               ('y', 0.00299, 0.98286),
>               ('z', 0.00477, 0.99905)
>              ]


> foo :: Float -> String
> foo x = printf "%.5f" x


> probSwitch i prob = mkFilter "ProbSwitch" [i, prob] 1

> findConsonant c = sndThd $ head $ filter (\(x, _, _) -> x == c) $ consonants
>     where sndThd (_, y, z) = (y, z)



> playWord :: Int -> String -> IO ()
> playWord _ [] = return ()
> playWord i s = do withSC3 $ \fd -> (send fd $ c_set [(offset i 15, 1)])
>                   playLetters i s
>                   withSC3 $ \fd -> (send fd $ c_set [(offset i 15, 0)])
>                   playLetter i 'a'
>  --                 playLetter i 'y'


> playLetters :: Int -> String -> IO ()
> playLetters _ [] = return ()
> playLetters i (x:xs) = do playLetter i x
>                           threadDelay (50000) -- 10th of second
>                           playLetters i xs

> playLetter i x | isConsonant x = playConsonant i x 0.25
>                | otherwise = playVowel i x 

> isConsonant :: Char -> Bool
> isConsonant = not . isVowel

> isVowel :: Char -> Bool
> isVowel x = x `elem` "aeiou"

> playConsonant i c lagTime =
>     do let (delay, blend) = findConsonant c
>        withSC3 $ \fd -> (send fd $ c_set [((offset i 16), lagTime), 
>                                           ((offset i 17), delay), 
>                                           ((offset i 18), blend)
>                                           ]
>                          )

> playVowel i c = sendVowel i $ 'a':(Char.toUpper c):[]

> ks i = do wn <- whiteNoise AR
>           let (gain, delay, blend) = (in' 1 KR (offsetU i 15), 
>                                       in' 1 KR (offsetU i 17), 
>                                       in' 1 KR (offsetU i 18)
>                                      )
>               ugenIf a b c = (a * b) + ((1 - a) * c)
>               wn0 = (wn / 2) + 0.5
>               probSwitch i prob = ugenIf (wn0 >* prob) i (negate i)
>               laggedDelay = lag delay (in' 1 KR (offsetU i 16))
>               laggedBlend = lag blend (in' 1 KR (offsetU i 16))
>               vFilter i f a b = resonz i f (b / f) * dbAmp a
>               n = sinOsc AR 200 0
>               a0 = decay (impulse AR 200 0) 0.0025 * n
>               a1 = (localIn 1 AR + (a0 * gain))
>               a2 = delayN a1 0.01 laggedDelay
>               a3 = delay1 a2
>               a4 = (a2 + a3) / 2.0
>               a5 = probSwitch a4 blend
>               a6 = mix (vFilter a5 
>                         (in' 5 KR (offsetU i 0)) 
>                         (in' 5 KR (offsetU i 5)) 
>                         (in' 5 KR (offsetU i 10))
>                        )
>               a7 = a6 * 1.5
>           return $ MRG [localOut (a5 * 0.99), out 0 (MCE [a7, a7])]


> offset :: Int -> Int -> Int
> offset i n = fromIntegral $ (i * 20) + n

> offsetU :: Int -> Int -> UGen
> offsetU i n = Constant $ fromIntegral $ offset i n
