(c) C. Alex McLean 2007
Released under GPL version 2.  See the COPYING file or gnu.org for details.

> module Parse where
    
> import Data.List
> import Text.ParserCombinators.Parsec
> import qualified Text.ParserCombinators.Parsec.Token as P
> import Text.ParserCombinators.Parsec.Language ( haskellDef )

> import Music

> lexer      = P.makeTokenParser haskellDef
> lexeme     = P.lexeme lexer
> whiteSpace = P.whiteSpace lexer
> braces     = P.braces lexer
> brackets   = P.brackets lexer
> symbol     = P.symbol lexer
> natural    = P.natural lexer

> parseMusic :: String -> Either ParseError SStructure
> parseMusic input = parse pMusic "" input

> pMusic :: Parser SStructure
> pMusic = do whiteSpace
>             pCycle

> pCycle :: Parser SStructure
> pCycle = do xs <- pPart `sepBy` (skipMany space)
>             return (Cycle $ concat xs)

Returns a list of structures because integers are expanded into
multiple Atom structures.

> pPart :: Parser [SStructure]
> pPart = pAtom
>         <|> pPolyrhythm
>         <|> pPolymetry

> pAtom :: Parser [SStructure]
> pAtom = pSound
>         <|> pRest

> pSound :: Parser [SStructure]
> pSound = do w <- lexeme (many1 letter) <?> "vocable word"
>             return [Atom $ Sound w]

> pRest :: Parser [SStructure]
> pRest = do n <- natural
>            return $ replicate (fromInteger n) $ Atom Rest
>         <|>
>         do (Parse.symbol "_")
>            return [Atom Rest]

> pPoly :: Parser [SStructure]
> pPoly  = pCycle `sepBy` (Parse.symbol ",")

> pPolyrhythm :: Parser [SStructure]
> pPolyrhythm = do xs <- Parse.braces pPoly
>                  return [expand $ Polymetry xs]

> pPolymetry :: Parser [SStructure]
> pPolymetry = do xs <- Parse.brackets pPoly
>                 return [fill $ Polymetry xs ]
