(c) C. Alex McLean 2007
Released under GPL version 2.  See the COPYING file or gnu.org for details.

> module Player where

> import Control.Concurrent
> import Control.Concurrent.MVar
> import System.Time 

> import Music
> import MyTime
> import KarplusStrong (playWord, initSynth)

> player :: MVar [[Event String]] -> IO ()
> player musicM = do start <- getClockTime
>                    player' musicM start 0

> player' :: MVar [[Event String]] -> ClockTime -> Int -> IO ()
> player' musicM start points = 
>     do events <- readMVar musicM
>        points' <- sendEvents start points events
>        player' musicM start points'


> sendEvents :: ClockTime -> Int -> [[Event String]] -> IO Int
> sendEvents start points [] = return points
> sendEvents start points xs = do sequence_ $ zipWith (sendPoint start points) [0..] xs
>                                 return $ points + length xs

> sendPoint :: ClockTime -> Int -> Int -> [Event String] -> IO ()
> sendPoint start points offset events
>     = do Player.wait start (points + offset)
>          sequence_ $ zipWith sendEvent [0..] events


> sendEvent voice (Sound s) = 
>     do forkIO $ playWord (voice `mod` voices) s
>        return ()
> sendEvent _ _ = return ()

> wait start n = do now <- getClockTime
>                   let micros = (n * delay - (microDiff now start))
>                   if micros > 0 then (threadDelay micros) else return ()
>                   return ()

> mapN :: ((Int, a) -> b) -> [a] -> [b]
> mapN f xs = map f $ zip [0 ..] xs

> voices = 6

> initPlayer :: IO ()
> initPlayer = initSynth voices
