(c) C. Alex McLean 2007
Released under GPL version 2.  See the COPYING file or gnu.org for details.

> module Util where

> import Data.List (group, sort, nub, groupBy, sortBy, (\\))
> import Safe

> tallySnd :: (Eq a) => (a, [a]) -> (a, [(a, Int)])
> tallySnd x = (fst x, tally (snd x))

> tally :: Eq a => [a] -> [(a, Int)]
> tally xs = map (\x -> (head x, length x)) $ group xs

> groupByFst :: (Eq a) => [(a, b)] -> [(a, [b])]
> groupByFst xs = map (\x -> (fst $ head x, map snd x)) $ groupBy (\x y -> (fst x) == (fst y)) xs

> groupBySnd :: (Eq b, Ord b) => [(a, b)] -> [([a], b)]
> groupBySnd xs = map (\x -> (map fst x, snd $ head x)) $ groupBy (\x y -> (snd x) == (snd y)) $ sortBy (\x y -> (snd x) `compare` (snd y)) xs

> sortByFst :: Ord a => [(a,b)] -> [(a,b)]
> sortByFst = sortBy (\ x y -> compare (fst x) (fst y))

> bigramMaybe :: [a] -> [(Maybe a, Maybe a)]
> bigramMaybe [] = []
> bigramMaybe xs = (Nothing, Just (head xs)) : bigramMaybe' xs

> bigramMaybe' :: [a] -> [(Maybe a, Maybe a)]
> bigramMaybe' xs | length xs < 2 = [(Just (atNote "bigramMaybe" xs 0), Nothing)]
>                 | otherwise = (Just (atNote "bigramMaybe" xs 0), Just (atNote "bigramMaybe" xs 1)) : (bigramMaybe' $ tail xs)

> bigram :: [a] -> [(a, a)]
> bigram [] = []
> bigram (x:[]) = []
> bigram xs = ((atNote "bigram" xs 0), (atNote "bigram" xs 1)) : (bigram $ tail xs)

> circDiffBigram :: [Int] -> [(Int, Int)]
> circDiffBigram xs = (snd $ last b, fst $ head b) : b
>     where b = diffBigram xs

> diffBigram :: [Int] -> [(Int, Int)]
> diffBigram xs = bigram $ map (\(a, b) -> b - a) $ bigram xs

Find the positive integers that the given positive integer is
divisible by, including itself and 1.

> divs x = filter (\y -> (x `mod` y) == 0) [1 .. x]

> rights = map fromRight . filter isRight

> fromRight (Right a) = a

> isRight (Right _) = True
> isRight _ = False

> justPairs :: [(a, b)] -> [(Maybe a, Maybe b)]
> justPairs = map (\(x, y) -> (Just x, Just y))

> nothingHead :: [(Maybe a, Maybe a)] -> [(Maybe a, Maybe a)]
> nothingHead xs@(x:_) = (Nothing, fst x):xs

> atD :: Double -> [a] -> a
> atD f xs = xs !! (floor $ f * (fromIntegral $ length xs))
